@@ -1,18 +1,41 @@
 from __future__ import annotations
 
+import logging
 from dataclasses import dataclass, field
+from enum import Enum
 from pathlib import Path
 
 from dataclass_binder import Binder
 
 
+class InstallMethod(Enum):
+    LINK = "link"
+    COPY = "copy"
+
+    @staticmethod
+    def from_string(install_method: str | None) -> InstallMethod:
+        if not install_method:
+            return InstallMethod.LINK
+
+        try:
+            return InstallMethod(install_method)
+        except ValueError:
+            logging.exception("could not determine install method")
+
+        return InstallMethod.LINK
+
+
 @dataclass
 class PkgConfig:
     name: str
     url: str
     package_root: str
     updates_disabled: bool = False
-    branch: str = None
+    branch: str | None = None
+    install_method: str | None = None
+
+    def get_install_method(self) -> InstallMethod:
+        return InstallMethod.from_string(self.install_method)
 
 
 @dataclass
@@ -44,6 +67,8 @@ class Config:
                     lines.append("updates-disabled = true")
                 if pkg.branch:
                     lines.append(f'branch = "{pkg.branch}"')
+                if pkg.get_install_method() != InstallMethod.LINK:
+                    lines.append(f'install-method = "{pkg.install_method}"')
                 lines.append("")
 
         for dest in self.destinations:
