@@ -1,3 +1,4 @@
+import sys
 from pathlib import Path
 
 from gitpkg.config import Destination, PkgConfig
@@ -12,6 +13,10 @@ class GitPkgError(Exception):
         self.raw_message = message
         super().__init__(f"ERROR: git pkg: {message}")
 
+    @classmethod
+    def create(cls, message: str):
+        return cls(message)
+
 
 class DestinationWithNameAlreadyExistsError(GitPkgError):
     def __init__(self, name: str):
@@ -62,7 +67,9 @@ class UnknownPackageError(GitPkgError):
 
 
 class PackageUrlChangedError(GitPkgError):
-    def __init__(self, destination: Destination, ref_pkg: PkgConfig, pkg: PkgConfig):
+    def __init__(
+        self, destination: Destination, ref_pkg: PkgConfig, pkg: PkgConfig
+    ):
         super().__init__(
             f"package '{pkg.name}' (dest: '{destination.name}') changed url "
             f"from '{ref_pkg.url}' to '{pkg.url}', please uninstall the "
@@ -81,3 +88,8 @@ class PackageRootDirNotFoundError(GitPkgError):
 class NameInvalidError(GitPkgError):
     def __init__(self, name: str):
         super().__init__(f"Name '{name}' is not valid!")
+
+
+class NotSupportedByPlatformError(GitPkgError):
+    def __init__(self, msg: str):
+        super().__init__(f"NOT SUPPORTED ON '{sys.platform}': {msg}")
