@@ -8,22 +8,30 @@ import re
 import shutil
 from dataclasses import dataclass
 from datetime import datetime
+from filecmp import dircmp
 from pathlib import Path
 
 from git import FetchInfo, GitConfigParser, Repo
 
-from gitpkg.config import Config, Destination, PkgConfig
+from gitpkg.config import Config, Destination, InstallMethod, PkgConfig
 from gitpkg.errors import (
     DestinationWithNameAlreadyExistsError,
     DestinationWithPathAlreadyExistsError,
     NameInvalidError,
+    NotSupportedByPlatformError,
     PackageAlreadyInstalledError,
     PackageRootDirNotFoundError,
     PackageUrlChangedError,
     PkgHasAlreadyBeenAddedError,
     UnknownPackageError,
 )
-from gitpkg.utils import extract_repository_name_from_url, symlink_exists
+from gitpkg.utils import (
+    does_actually_exist,
+    extract_repository_name_from_url,
+    is_symlink,
+    is_windows,
+    safe_dir_delete,
+)
 
 _GITPKGS_DIR = ".gitpkgs"
 _CONFIG_FILE = ".gitpkg.toml"
@@ -49,7 +57,9 @@ class PkgManager:
                 return dest
         return None
 
-    def find_packages_by_destination(self, destination: Destination) -> list[PkgConfig]:
+    def find_packages_by_destination(
+        self, destination: Destination
+    ) -> list[PkgConfig]:
         """Find all packages associated with given destination"""
         if destination.name not in self._config.packages:
             return []
@@ -125,7 +135,8 @@ class PkgManager:
         self._write_config()
 
     def remove_package(self, destination: Destination, pkg: PkgConfig) -> None:
-        """Remove package from destination in config (does not uninstall package)"""
+        """Remove package from destination in config
+        (does not uninstall package)"""
 
         if not self.is_package_registered(destination, pkg):
             raise UnknownPackageError(destination, pkg)
@@ -157,12 +168,16 @@ class PkgManager:
 
         return (
             (self.project_root_directory() / ".gitmodules").exists()
-            and symlink_exists(self.package_install_location(destination, pkg))
+            and does_actually_exist(
+                self.package_install_location(destination, pkg)
+            )
             and self._get_pkg_submodule_location(destination, pkg).exists()
             and self._gitmodules_internal_location(destination, pkg).exists()
         )
 
-    def find_package(self, destination: Destination, pkg_name: str) -> PkgConfig | None:
+    def find_package(
+        self, destination: Destination, pkg_name: str
+    ) -> PkgConfig | None:
         """Find a package in a destination by name"""
         if destination.name not in self._config.packages:
             return None
@@ -174,7 +189,8 @@ class PkgManager:
     def has_package_config_been_changed(
         self, destination: Destination, pkg: PkgConfig
     ) -> bool:
-        """Is the given package configuration different from what's installed?"""
+        """Is the given package configuration different from
+        what's installed?"""
         ref_pkg = self.find_package(destination, pkg.name)
         if ref_pkg is None:
             raise UnknownPackageError(destination, pkg)
@@ -193,7 +209,9 @@ class PkgManager:
 
         return self._has_repo_changed(destination, pkg)
 
-    def _has_repo_changed(self, destination: Destination, pkg: PkgConfig) -> bool:
+    def _has_repo_changed(
+        self, destination: Destination, pkg: PkgConfig
+    ) -> bool:
         """Has the physical repository changed?"""
         # no changes in config found, next test against the actual repo...
         if pkg.branch:
@@ -204,20 +222,61 @@ class PkgManager:
                     f"{ref_repo.active_branch.name}, but package "
                     f"wanted: {pkg.branch}"
                 )
-                return False
+                return True
 
         pkg_path = self.package_install_location(destination, pkg)
 
-        if pkg.package_root and symlink_exists(pkg_path) and pkg_path.is_symlink():
+        # was installed as link but is now copy
+        if pkg.get_install_method() == InstallMethod.COPY and is_symlink(
+            pkg_path
+        ):
+            return True
+
+        # was installed as copy and is now link
+        if pkg.get_install_method() == InstallMethod.LINK and not is_symlink(
+            pkg_path
+        ):
+            return True
+
+        # check if contents of directories are different
+        if (
+            pkg.get_install_method() == InstallMethod.COPY
+            and pkg.package_root
+            and does_actually_exist(pkg_path)
+        ):
             source_path = (
-                self._get_pkg_submodule_location(destination, pkg) / pkg.package_root
+                self._get_pkg_submodule_location(destination, pkg)
+                / pkg.package_root
             )
-            target_path = Path(os.readlink(pkg_path))
+
+            if source_path.exists():
+                result = dircmp(source_path, pkg_path)
+
+                if len(result.left_only) > 0:
+                    return True
+
+                if len(result.right_only) > 0:
+                    return True
+
+                if len(result.diff_files) > 0:
+                    return True
+
+                return len(result.funny_files) > 0
+
+        # check if package root is different in a link setting
+        if pkg.package_root and is_symlink(pkg_path):
+            source_path = (
+                self._get_pkg_submodule_location(destination, pkg)
+                / pkg.package_root
+            )
+            target_path = pkg_path.readlink()
 
             # this path should be relative, so we apply it on top of the install
             # location
             if not target_path.is_absolute():
-                vendor_dir = self.package_install_location(destination, pkg).parent
+                vendor_dir = self.package_install_location(
+                    destination, pkg
+                ).parent
                 target_path = vendor_dir / target_path
 
             # link target does not exist? Something changed!
@@ -225,7 +284,8 @@ class PkgManager:
                 return True
 
             logging.debug(
-                f"package root: Source is {source_path}, " f"Target is: {target_path}"
+                f"package root: Source is {source_path}, "
+                f"Target is: {target_path}"
             )
             return not source_path.samefile(target_path)
 
@@ -259,16 +319,16 @@ class PkgManager:
         install_dir.parent.mkdir(parents=True, exist_ok=True)
 
         # delete if install dir is there (missing links are exists = False)
-        install_dir.unlink(missing_ok=True)
+        safe_dir_delete(install_dir)
 
         if not repo_used_by_other_pkg and submodule_location.exists():
-            shutil.rmtree(submodule_location)
+            safe_dir_delete(submodule_location)
 
         gitmodules_file = self.project_root_directory() / ".gitmodules"
 
         internal_dir = self._gitmodules_internal_location(destination, pkg)
         if internal_dir.exists() and not gitmodules_file.exists():
-            shutil.rmtree(internal_dir)
+            safe_dir_delete(internal_dir)
 
         if not submodule_location.exists():
             if internal_dir.exists():
@@ -277,7 +337,7 @@ class PkgManager:
                 gitdir = submodule_location / ".git"
 
                 if gitdir.exists():
-                    shutil.rmtree(gitdir)
+                    safe_dir_delete(gitdir)
 
                 rel_path = os.path.relpath(internal_dir, submodule_location)
                 gitdir.write_text(f"gitdir: {rel_path}")
@@ -295,31 +355,43 @@ class PkgManager:
         if not pkg_package_root_dir.exists():
             raise PackageRootDirNotFoundError(pkg, pkg_package_root_dir)
 
-        if symlink_exists(install_dir):
+        if does_actually_exist(install_dir):
             install_dir.unlink()
 
-        install_dir.symlink_to(
-            os.path.relpath(pkg_package_root_dir, install_dir.parent)
-        )
+        match pkg.get_install_method():
+            case InstallMethod.LINK:
+                if is_windows():
+                    msg = "Install method 'link' is not supported on Windows!"
+                    raise NotSupportedByPlatformError(msg)
+                install_dir.symlink_to(
+                    os.path.relpath(pkg_package_root_dir, install_dir.parent)
+                )
+            case InstallMethod.COPY:
+                shutil.copytree(pkg_package_root_dir, install_dir)
+            case method:
+                msg = f"Unknown install method {method}"
+                raise ValueError(msg)
 
         logging.debug(f"installed package '{pkg.name}' to {install_dir}")
 
-    def uninstall_package(self, destination: Destination, pkg: PkgConfig) -> None:
+    def uninstall_package(
+        self, destination: Destination, pkg: PkgConfig
+    ) -> None:
         """Uninstalls the package from disk"""
 
         repo_used_by_other_pkg = self._repo_used_by_other_pkg(destination, pkg)
 
         internal_dir = self._gitmodules_internal_location(destination, pkg)
         if not repo_used_by_other_pkg and internal_dir.exists():
-            shutil.rmtree(internal_dir)
+            safe_dir_delete(internal_dir)
 
         install_dir = self.package_install_location(destination, pkg)
-        if symlink_exists(install_dir):
-            install_dir.unlink()
+        if does_actually_exist(install_dir):
+            safe_dir_delete(install_dir)
 
         submodule_location = self._get_pkg_submodule_location(destination, pkg)
         if not repo_used_by_other_pkg and submodule_location.exists():
-            shutil.rmtree(submodule_location)
+            safe_dir_delete(submodule_location)
 
         if not repo_used_by_other_pkg:
             self._remove_pkg_from_gitmodules(destination, pkg)
@@ -328,7 +400,8 @@ class PkgManager:
             self.remove_package(destination, pkg)
 
         logging.debug(
-            f"uninstalled package '{pkg.name}' from dest: " f"'{destination.name}'"
+            f"uninstalled package '{pkg.name}' from dest: "
+            f"'{destination.name}'"
         )
 
     def update_package(
@@ -371,11 +444,17 @@ class PkgManager:
             return PkgUpdateResult.NO_UPDATE_AVAILABLE
 
         if fetch_info.commit != fetch_info.old_commit:
-            return (
-                PkgUpdateResult.UPDATED
-                if not check_only
-                else PkgUpdateResult.UPDATE_AVAILABLE
-            )
+            if check_only:
+                return PkgUpdateResult.UPDATE_AVAILABLE
+
+            # since we dont have a link in install method copy we just delete
+            # and re-copy the folder
+            if pkg.get_install_method() == InstallMethod.COPY:
+                install_dir = self.package_install_location(destination, pkg)
+                safe_dir_delete(install_dir)
+                shutil.copytree(submodule_location, install_dir)
+
+            return PkgUpdateResult.UPDATED
 
         return PkgUpdateResult.NO_UPDATE_AVAILABLE
 
@@ -398,7 +477,7 @@ class PkgManager:
             if dir.name in packages:
                 continue
             logging.debug(f"CLEAN: remove unused gitpkg {dir}")
-            shutil.rmtree(dir)
+            safe_dir_delete(dir)
 
         # remove links that point nowhere from dest dirs
         for dest in self.destinations():
@@ -407,8 +486,8 @@ class PkgManager:
             for dest_dir in dest_path.glob("*"):
                 if not dest_dir.is_symlink():
                     continue
-                target_dir = (dest_dir.parent / os.readlink(dest_dir)).resolve()
-                if not symlink_exists(target_dir):
+                target_dir = (dest_dir.parent / dest_dir.readlink()).resolve()
+                if not does_actually_exist(target_dir):
                     logging.debug(f"CLEAN: remove symlink {dest_dir}")
                     dest_dir.unlink()
 
@@ -450,17 +529,22 @@ class PkgManager:
                 pkg_ident = matches[0]
 
                 internal_dir = (
-                    self.project_root_directory() / ".git" / "modules" / pkg_ident
+                    self.project_root_directory()
+                    / ".git"
+                    / "modules"
+                    / pkg_ident
                 )
 
                 if internal_dir.exists():
                     logging.debug(f"CLEAN: remove internal dir {internal_dir}")
-                    shutil.rmtree(internal_dir)
+                    safe_dir_delete(internal_dir)
 
                 cp.remove_section(section)
             cp.write()
 
-    def _repo_used_by_other_pkg(self, dest: Destination, pkg: PkgConfig) -> bool:
+    def _repo_used_by_other_pkg(
+        self, dest: Destination, pkg: PkgConfig
+    ) -> bool:
         package_ident = self.package_identifier(dest, pkg)
 
         for ref_dest in self.destinations():
@@ -494,7 +578,9 @@ class PkgManager:
             if len(text.strip()) == 0:
                 gitmodules_file.unlink()
 
-    def _update_gitmodules_file(self, destination: Destination, pkg: PkgConfig) -> None:
+    def _update_gitmodules_file(
+        self, destination: Destination, pkg: PkgConfig
+    ) -> None:
         pkg_ident = self.package_identifier(destination, pkg)
         gitmodules_file = self.project_root_directory() / ".gitmodules"
         section = f'submodule "{pkg_ident}"'
@@ -536,7 +622,9 @@ class PkgManager:
         pkg: PkgConfig,
     ) -> Path:
         """Submodule location of package"""
-        return self._gitpkgs_location() / self.package_identifier(destination, pkg)
+        return self._gitpkgs_location() / self.package_identifier(
+            destination, pkg
+        )
 
     def package_identifier(self, _dest: Destination, pkg: PkgConfig) -> str:
         """Unique package identifier"""
@@ -569,7 +657,7 @@ class PkgManager:
         )
 
     @staticmethod
-    def from_environment():
+    def from_environment() -> PkgManager:
         """Create a package manager for the nearest git project"""
         repo = Repo(Path.cwd(), search_parent_directories=True)
         config = Config()
@@ -581,6 +669,22 @@ class PkgManager:
 
         return PkgManager(repo, config)
 
+    @staticmethod
+    def from_path(path: Path) -> PkgManager:
+        """Create a package manager for the specified path"""
+        if not path.exists():
+            raise FileNotFoundError
+
+        repo = Repo(path, search_parent_directories=False)
+        config = Config()
+
+        config_file = PkgManager._project_root_directory(repo) / _CONFIG_FILE
+
+        if config_file.exists():
+            config = Config.from_path(config_file)
+
+        return PkgManager(repo, config)
+
     @staticmethod
     def _project_root_directory(repo: Repo) -> Path:
         return Path(repo.git_dir).parent
